@@ -30,6 +30,7 @@
 #include "protocols/xdg-output-unstable-v1-client.h"
 
 #include "libxfce4windowing-private.h"
+#include "xfw-gdk-private.h"
 #include "xfw-monitor-private.h"
 #include "xfw-monitor-wayland.h"
 #include "xfw-monitor.h"
@@ -50,6 +51,7 @@ struct _XfwMonitorWayland {
     XfwMonitor parent;
 
     struct wl_output *output;
+    uint32_t global_name;
     struct zxdg_output_v1 *xdg_output;
 
     GdkRectangle physical_geometry;
@@ -57,6 +59,10 @@ struct _XfwMonitorWayland {
 
     guint32 output_dones : 4,
         xdg_output_done : 1;
+
+    XfwScreen *screen;
+    guint finalize_output_id;
+    guint finalize_output_count;
 };
 
 typedef struct {
@@ -96,6 +102,10 @@ xfw_monitor_wayland_finalize(GObject *object) {
         }
     }
 
+    if (monitor->finalize_output_id != 0) {
+        g_source_remove(monitor->finalize_output_id);
+    }
+
     G_OBJECT_CLASS(xfw_monitor_wayland_parent_class)->finalize(object);
 }
 
@@ -305,9 +315,33 @@ unscale_monitor_coordinates(GList *monitors, XfwMonitor *monitor) {
     g_array_free(found_y_segments, TRUE);
 }
 
-static void
-finalize_output(XfwMonitorManagerWayland *monitor_manager, XfwMonitorWayland *monitor_wl) {
-    g_debug("finalizing for output ID %d", wl_proxy_get_id((struct wl_proxy *)monitor_wl->output));
+static gboolean
+finalize_output(gpointer data) {
+    XfwMonitorWayland *monitor_wl = data;
+    GdkDisplay *display = gdk_display_get_default();
+    gint n_monitors = gdk_display_get_n_monitors(display);
+    gboolean connectors_set = TRUE;
+    for (gint i = 0; i < n_monitors; i++) {
+        GdkMonitor *gdkmonitor = gdk_display_get_monitor(display, i);
+        if (xfw_gdk_monitor_get_connector(gdkmonitor) == NULL) {
+            connectors_set = FALSE;
+            break;
+        }
+    }
+    if (connectors_set) {
+        g_debug("finalizing for output ID %d", wl_proxy_get_id((struct wl_proxy *)monitor_wl->output));
+    } else {
+        if (monitor_wl->finalize_output_count++ < 10) {
+            g_debug("delaying finalization of output ID %d because of missing gdk data", wl_proxy_get_id((struct wl_proxy *)monitor_wl->output));
+            if (monitor_wl->finalize_output_id != 0) {
+                g_source_remove(monitor_wl->finalize_output_id);
+            }
+            monitor_wl->finalize_output_id = g_timeout_add(100, finalize_output, monitor_wl);
+            return FALSE;
+        } else {
+            g_debug("can't wait for gdk data any longer, finalizing for output ID %d", wl_proxy_get_id((struct wl_proxy *)monitor_wl->output));
+        }
+    }
 
     XfwMonitor *monitor = XFW_MONITOR(monitor_wl);
 
@@ -358,11 +392,11 @@ finalize_output(XfwMonitorManagerWayland *monitor_manager, XfwMonitorWayland *mo
     };
     _xfw_monitor_set_workarea(monitor, &workarea);
 
-    GList added = { NULL, NULL, NULL };
-    GList *monitors = _xfw_screen_steal_monitors(monitor_manager->screen);
+    GList *added = NULL;
+    GList *monitors = _xfw_screen_steal_monitors(monitor_wl->screen);
     if (!g_list_find(monitors, monitor)) {
         monitors = g_list_append(monitors, g_object_ref(monitor));
-        added.data = monitor;
+        added = g_list_append(added, monitor);
     }
 
     // The compositor doesn't appear to tell us the monitor layout coordinates
@@ -420,7 +454,12 @@ finalize_output(XfwMonitorManagerWayland *monitor_manager, XfwMonitorWayland *mo
         _xfw_monitor_set_is_primary(a_monitor, a_monitor == primary_monitor);
     }
 
-    _xfw_screen_set_monitors(monitor_manager->screen, monitors, &added, NULL);
+    _xfw_screen_set_monitors(monitor_wl->screen, monitors, added, NULL);
+    g_list_free(added);
+
+    monitor_wl->finalize_output_count = 0;
+    monitor_wl->finalize_output_id = 0;
+    return FALSE;
 }
 
 static void
@@ -490,17 +529,20 @@ output_done(void *data, struct wl_output *output) {
     g_debug("output done for ID %d", wl_proxy_get_id((struct wl_proxy *)output));
     XfwMonitorManagerWayland *monitor_manager = data;
     XfwMonitorWayland *monitor = g_hash_table_lookup(monitor_manager->outputs_to_monitors, output);
+    monitor->screen = monitor_manager->screen;
     monitor->output_dones++;
 
-    if (monitor_manager->xdg_output_manager == NULL
-        || (wl_proxy_get_version((struct wl_proxy *)monitor_manager->xdg_output_manager) >= 3 && monitor->output_dones >= 2)
-        || monitor->xdg_output_done)
+    if (monitor->finalize_output_id == 0
+        && (monitor_manager->xdg_output_manager == NULL
+            || monitor->xdg_output_done
+            || (wl_proxy_get_version((struct wl_proxy *)monitor_manager->xdg_output_manager) >= 3
+                && monitor->output_dones >= 2)))
     {
         g_debug("finalizing output because: xdg_op_mgr=%p, xdg_op_mgr_vers=%d, xdg_op_done=%d",
                 monitor_manager->xdg_output_manager,
                 monitor_manager->xdg_output_manager != NULL ? (int)wl_proxy_get_version((struct wl_proxy *)monitor_manager->xdg_output_manager) : -1,
                 monitor->xdg_output_done);
-        finalize_output(monitor_manager, monitor);
+        finalize_output(monitor);
     }
 }
 
@@ -552,10 +594,14 @@ xdg_output_done(void *data, struct zxdg_output_v1 *xdg_output) {
     g_debug("xdg output done for ID %d", wl_proxy_get_id((struct wl_proxy *)xdg_output));
     XfwMonitorManagerWayland *monitor_manager = data;
     XfwMonitorWayland *monitor = g_hash_table_lookup(monitor_manager->xdg_outputs_to_monitors, xdg_output);
+    monitor->screen = monitor_manager->screen;
     monitor->xdg_output_done = TRUE;
 
-    if (monitor->output_dones > 0 && wl_proxy_get_version((struct wl_proxy *)monitor_manager->xdg_output_manager) < 3) {
-        finalize_output(monitor_manager, monitor);
+    if (monitor->finalize_output_id == 0
+        && monitor->output_dones > 0
+        && wl_proxy_get_version((struct wl_proxy *)monitor_manager->xdg_output_manager) < 3)
+    {
+        finalize_output(monitor);
     }
 }
 
@@ -591,9 +637,10 @@ _xfw_monitor_manager_wayland_new(XfwScreenWayland *wscreen) {
 }
 
 void
-_xfw_monitor_manager_wayland_new_output(XfwMonitorManagerWayland *monitor_manager, struct wl_output *output) {
+_xfw_monitor_manager_wayland_new_output(XfwMonitorManagerWayland *monitor_manager, struct wl_output *output, uint32_t global_name) {
     XfwMonitorWayland *monitor = g_object_new(XFW_TYPE_MONITOR_WAYLAND, NULL);
     monitor->output = output;
+    monitor->global_name = global_name;
 
     wl_output_add_listener(output, &output_listener, monitor_manager);
     g_hash_table_insert(monitor_manager->outputs_to_monitors, output, monitor);
@@ -604,25 +651,20 @@ _xfw_monitor_manager_wayland_new_output(XfwMonitorManagerWayland *monitor_manage
 }
 
 void
-_xfw_monitor_manager_wayland_global_removed(XfwMonitorManagerWayland *monitor_manager, uint32_t id) {
+_xfw_monitor_manager_wayland_global_removed(XfwMonitorManagerWayland *monitor_manager, uint32_t name) {
     GHashTableIter iter;
     g_hash_table_iter_init(&iter, monitor_manager->outputs_to_monitors);
 
     struct wl_output *output;
     XfwMonitorWayland *monitor;
     while (g_hash_table_iter_next(&iter, (gpointer)&output, (gpointer)&monitor)) {
-        if (wl_proxy_get_id((struct wl_proxy *)output) == id) {
-            if (monitor->xdg_output != NULL) {
-                g_hash_table_remove(monitor_manager->xdg_outputs_to_monitors, monitor->xdg_output);
-            }
-            g_hash_table_remove(monitor_manager->outputs_to_monitors, output);
-
-            GList removed = { NULL, NULL, NULL };
+        if (monitor->global_name == name) {
+            GList *removed = NULL;
             GList *monitors = _xfw_screen_steal_monitors(monitor_manager->screen);
             GList *lm = g_list_find(monitors, monitor);
             if (lm != NULL) {
                 monitors = g_list_delete_link(monitors, lm);
-                removed.data = monitor;
+                removed = g_list_append(removed, monitor);
 
                 XfwMonitor *primary_monitor = _xfw_monitor_guess_primary_monitor(monitors);
                 for (GList *l = monitors; l != NULL; l = l->next) {
@@ -631,12 +673,21 @@ _xfw_monitor_manager_wayland_global_removed(XfwMonitorManagerWayland *monitor_ma
                 }
             }
 
-            _xfw_screen_set_monitors(monitor_manager->screen, monitors, NULL, &removed);
+            _xfw_screen_set_monitors(monitor_manager->screen, monitors, NULL, removed);
 
-            if (removed.data != NULL) {
-                g_object_unref(XFW_MONITOR(removed.data));
+            if (removed != NULL) {
+                g_object_unref(XFW_MONITOR(removed->data));
+                g_list_free(removed);
             }
 
+            if (monitor->xdg_output != NULL) {
+                g_hash_table_remove(monitor_manager->xdg_outputs_to_monitors, monitor->xdg_output);
+                g_clear_pointer(&monitor->xdg_output, zxdg_output_v1_destroy);
+            }
+
+            gboolean has_release = wl_proxy_get_version((struct wl_proxy *)monitor->output) >= WL_OUTPUT_RELEASE_SINCE_VERSION;
+            g_clear_pointer(&monitor->output, has_release ? wl_output_release : wl_output_destroy);
+            g_hash_table_iter_remove(&iter);
             break;
         }
     }
