@@ -50,10 +50,14 @@ struct _XfwScreenWayland {
     GList *pending_seats;
 
     gboolean defer_toplevel_manager;
-    uint32_t toplevel_manager_id;
+    uint32_t toplevel_manager_name;
     uint32_t toplevel_manager_version;
     struct zwlr_foreign_toplevel_manager_v1 *toplevel_manager;
 
+    gboolean defer_workspace_manager;
+    uint32_t workspace_manager_name;
+    uint32_t workspace_manager_version;
+
     GList *windows;
     GList *windows_stacked;
     GHashTable *wl_windows;
@@ -76,9 +80,10 @@ static void show_desktop_disconnect(gpointer object, gpointer data);
 static void async_roundtrip_done(void *data, struct wl_callback *callback, uint32_t callback_id);
 
 static void init_toplevel_manager(XfwScreenWayland *screen);
+static void init_workspace_manager(XfwScreenWayland *screen);
 
-static void registry_global(void *data, struct wl_registry *registry, uint32_t id, const char *interface, uint32_t version);
-static void registry_global_remove(void *data, struct wl_registry *registry, uint32_t id);
+static void registry_global(void *data, struct wl_registry *registry, uint32_t name, const char *interface, uint32_t version);
+static void registry_global_remove(void *data, struct wl_registry *registry, uint32_t name);
 
 static void toplevel_manager_toplevel(void *data, struct zwlr_foreign_toplevel_manager_v1 *wl_manager, struct zwlr_foreign_toplevel_handle_v1 *wl_toplevel);
 static void toplevel_manager_finished(void *data, struct zwlr_foreign_toplevel_manager_v1 *wl_manager);
@@ -114,6 +119,7 @@ xfw_screen_wayland_class_init(XfwScreenWaylandClass *klass) {
 static void
 xfw_screen_wayland_init(XfwScreenWayland *screen) {
     screen->defer_toplevel_manager = TRUE;
+    screen->defer_workspace_manager = TRUE;
     screen->wl_windows = g_hash_table_new_full(g_direct_hash, g_direct_equal, NULL, g_object_unref);
 }
 
@@ -136,13 +142,17 @@ xfw_screen_wayland_constructed(GObject *obj) {
         wl_display_dispatch(wscreen->wl_display);
     }
 
-    // We defer binding to the toplevel manager until after we have all
+    // We defer binding to the toplevel and workspace managers until after we have all
     // XfwMonitor instances initialized.  Otherwise, we would get output_enter
-    // events for toplevels, but have no XfwMonitor to match them to.
+    // events for toplevels or workspace groups, but have no XfwMonitor to match them to.
     wscreen->defer_toplevel_manager = FALSE;
-    if (wscreen->toplevel_manager_id != 0 && wscreen->toplevel_manager_version != 0) {
+    if (wscreen->toplevel_manager_name != 0 && wscreen->toplevel_manager_version != 0) {
         init_toplevel_manager(wscreen);
     }
+    wscreen->defer_workspace_manager = FALSE;
+    if (wscreen->workspace_manager_name != 0 && wscreen->workspace_manager_version != 0) {
+        init_workspace_manager(wscreen);
+    }
 
     if (wscreen->toplevel_manager != NULL) {
         while (wscreen->async_roundtrips != NULL) {
@@ -152,7 +162,11 @@ xfw_screen_wayland_constructed(GObject *obj) {
         g_message("Your compositor does not support the wlr_foreign_toplevel_manager_v1 protocol");
     }
 
-    if (xfw_screen_get_workspace_manager(XFW_SCREEN(screen)) == NULL) {
+    if (xfw_screen_get_workspace_manager(XFW_SCREEN(screen)) != NULL) {
+        while (wscreen->async_roundtrips != NULL) {
+            wl_display_dispatch(wscreen->wl_display);
+        }
+    } else {
         g_message("Your compositor does not support the ext_workspace_manager_v1 protocol");
         _xfw_screen_set_workspace_manager(XFW_SCREEN(screen), _xfw_workspace_manager_dummy_new(screen));
     }
@@ -292,32 +306,48 @@ async_roundtrip_done(void *data, struct wl_callback *callback, uint32_t callback
 static void
 init_toplevel_manager(XfwScreenWayland *screen) {
     g_return_if_fail(!screen->defer_toplevel_manager);
-    g_return_if_fail(screen->toplevel_manager_id != 0);
+    g_return_if_fail(screen->toplevel_manager_name != 0);
     g_return_if_fail(screen->toplevel_manager_version != 0);
     g_return_if_fail(screen->toplevel_manager == NULL);
 
     screen->toplevel_manager = wl_registry_bind(screen->wl_registry,
-                                                screen->toplevel_manager_id,
+                                                screen->toplevel_manager_name,
                                                 &zwlr_foreign_toplevel_manager_v1_interface,
                                                 MIN(screen->toplevel_manager_version, 3));
     zwlr_foreign_toplevel_manager_v1_add_listener(screen->toplevel_manager, &toplevel_manager_listener, screen);
     add_async_roundtrip(screen);
 }
 
+static void
+init_workspace_manager(XfwScreenWayland *screen) {
+    g_return_if_fail(!screen->defer_workspace_manager);
+    g_return_if_fail(screen->workspace_manager_name != 0);
+    g_return_if_fail(screen->workspace_manager_version != 0);
+    g_return_if_fail(xfw_screen_get_workspace_manager(XFW_SCREEN(screen)) == NULL);
+
+    struct ext_workspace_manager_v1 *wl_workspace_manager = wl_registry_bind(screen->wl_registry,
+                                                                             screen->workspace_manager_name,
+                                                                             &ext_workspace_manager_v1_interface,
+                                                                             MIN((uint32_t)ext_workspace_manager_v1_interface.version,
+                                                                                 screen->workspace_manager_version));
+    _xfw_screen_set_workspace_manager(XFW_SCREEN(screen), _xfw_workspace_manager_wayland_new(screen, wl_workspace_manager));
+    add_async_roundtrip(screen);
+}
+
 
 static void
-registry_global(void *data, struct wl_registry *registry, uint32_t id, const char *interface, uint32_t version) {
+registry_global(void *data, struct wl_registry *registry, uint32_t name, const char *interface, uint32_t version) {
     XfwScreenWayland *wscreen = XFW_SCREEN_WAYLAND(data);
 
     if (strcmp(zwlr_foreign_toplevel_manager_v1_interface.name, interface) == 0) {
-        wscreen->toplevel_manager_id = id;
+        wscreen->toplevel_manager_name = name;
         wscreen->toplevel_manager_version = version;
         if (!wscreen->defer_toplevel_manager) {
             init_toplevel_manager(wscreen);
         }
     } else if (strcmp(wl_seat_interface.name, interface) == 0) {
-        struct wl_seat *wl_seat = wl_registry_bind(wscreen->wl_registry, id, &wl_seat_interface, 2);
-        XfwSeatWayland *seat = _xfw_seat_wayland_new(XFW_SCREEN(wscreen), wl_seat);
+        struct wl_seat *wl_seat = wl_registry_bind(wscreen->wl_registry, name, &wl_seat_interface, 2);
+        XfwSeatWayland *seat = _xfw_seat_wayland_new(XFW_SCREEN(wscreen), wl_seat, name);
         wscreen->pending_seats = g_list_prepend(wscreen->pending_seats, seat);
         add_async_roundtrip(wscreen);
     } else if (strcmp(ext_workspace_manager_v1_interface.name, interface) == 0) {
@@ -325,20 +355,19 @@ registry_global(void *data, struct wl_registry *registry, uint32_t id, const cha
         if (xfw_screen_get_workspace_manager(screen) != NULL) {
             g_message("Already have a workspace manager, but got a new ext_workspace_manager_v1 global");
         } else {
-            struct ext_workspace_manager_v1 *wl_workspace_manager = wl_registry_bind(registry,
-                                                                                     id,
-                                                                                     &ext_workspace_manager_v1_interface,
-                                                                                     MIN((uint32_t)ext_workspace_manager_v1_interface.version, version));
-            _xfw_screen_set_workspace_manager(screen, _xfw_workspace_manager_wayland_new(wscreen, wl_workspace_manager));
-            add_async_roundtrip(wscreen);
+            wscreen->workspace_manager_name = name;
+            wscreen->workspace_manager_version = version;
+            if (!wscreen->defer_workspace_manager) {
+                init_workspace_manager(wscreen);
+            }
         }
     } else if (strcmp(wl_output_interface.name, interface) == 0) {
-        struct wl_output *output = wl_registry_bind(registry, id, &wl_output_interface, MIN(version, 4));
-        _xfw_monitor_manager_wayland_new_output(wscreen->monitor_manager, output);
+        struct wl_output *output = wl_registry_bind(registry, name, &wl_output_interface, MIN(version, 4));
+        _xfw_monitor_manager_wayland_new_output(wscreen->monitor_manager, output, name);
         add_async_roundtrip(wscreen);
     } else if (strcmp(zxdg_output_manager_v1_interface.name, interface) == 0) {
         struct zxdg_output_manager_v1 *xdg_output_manager = wl_registry_bind(registry,
-                                                                             id,
+                                                                             name,
                                                                              &zxdg_output_manager_v1_interface,
                                                                              MIN(version, 3));
         _xfw_monitor_manager_wayland_new_xdg_output_manager(wscreen->monitor_manager, xdg_output_manager);
@@ -347,15 +376,15 @@ registry_global(void *data, struct wl_registry *registry, uint32_t id, const cha
 }
 
 static void
-registry_global_remove(void *data, struct wl_registry *registry, uint32_t id) {
+registry_global_remove(void *data, struct wl_registry *registry, uint32_t name) {
     XfwScreenWayland *screen = XFW_SCREEN_WAYLAND(data);
 
     gboolean seat_removed = FALSE;
 
     for (GList *l = xfw_screen_get_seats(XFW_SCREEN(screen)); l != NULL; l = l->next) {
         XfwSeatWayland *seat = XFW_SEAT_WAYLAND(l->data);
-        struct wl_seat *wl_seat = _xfw_seat_wayland_get_wl_seat(seat);
-        if (id == wl_proxy_get_id((struct wl_proxy *)wl_seat)) {
+        uint32_t global_name = _xfw_seat_wayland_get_global_name(seat);
+        if (name == global_name) {
             _xfw_screen_seat_removed(XFW_SCREEN(screen), XFW_SEAT(seat));
             seat_removed = TRUE;
             break;
@@ -365,8 +394,8 @@ registry_global_remove(void *data, struct wl_registry *registry, uint32_t id) {
     if (!seat_removed) {
         for (GList *l = screen->pending_seats; l != NULL; l = l->next) {
             XfwSeatWayland *seat = XFW_SEAT_WAYLAND(l->data);
-            struct wl_seat *wl_seat = _xfw_seat_wayland_get_wl_seat(seat);
-            if (id == wl_proxy_get_id((struct wl_proxy *)wl_seat)) {
+            uint32_t global_name = _xfw_seat_wayland_get_global_name(seat);
+            if (name == global_name) {
                 screen->pending_seats = g_list_delete_link(screen->pending_seats, l);
                 g_object_unref(seat);
                 seat_removed = TRUE;
@@ -376,7 +405,7 @@ registry_global_remove(void *data, struct wl_registry *registry, uint32_t id) {
     }
 
     if (!seat_removed) {
-        _xfw_monitor_manager_wayland_global_removed(screen->monitor_manager, id);
+        _xfw_monitor_manager_wayland_global_removed(screen->monitor_manager, name);
     }
 }
 
