@@ -59,11 +59,14 @@
 #include "xfw-screen-wayland.h"
 #endif
 
+#include "libxfce4windowing-visibility.h"
+
 #define XFW_SCREEN_GET_PRIVATE(screen) ((XfwScreenPrivate *)xfw_screen_get_instance_private((XfwScreen *)screen))
 #define GDK_SCREEN_XFW_SCREEN_KEY "libxfce4windowing-xfw-screen"
 
 typedef struct _XfwXcreenPrivate {
     GdkScreen *gdk_screen;
+    GList *seats;
     XfwWorkspaceManager *workspace_manager;
     GList *monitors;
     XfwMonitor *primary_monitor;
@@ -100,6 +103,42 @@ xfw_screen_class_init(XfwScreenClass *klass) {
     gobject_class->get_property = xfw_screen_get_property;
     gobject_class->finalize = xfw_screen_finalize;
 
+    /**
+     * XfwScreen::seat-added:
+     * @screen: the object which received the signal.
+     * @seat: the new #XfwSeat that was added.
+     *
+     * Emitted when @seat has been added to @screen.
+     **/
+    g_signal_new("seat-added",
+                 XFW_TYPE_SCREEN,
+                 G_SIGNAL_RUN_LAST,
+                 0,
+                 NULL,
+                 NULL,
+                 g_cclosure_marshal_VOID__OBJECT,
+                 G_TYPE_NONE,
+                 1,
+                 XFW_TYPE_SEAT);
+
+    /**
+     * XfwScreen::seat-removed:
+     * @screen: the object which received the signal.
+     * @seat: the #XfwSeat that was removed.
+     *
+     * Emitted when @seat has been added to @screen.
+     **/
+    g_signal_new("seat-removed",
+                 XFW_TYPE_SCREEN,
+                 G_SIGNAL_RUN_LAST,
+                 0,
+                 NULL,
+                 NULL,
+                 g_cclosure_marshal_VOID__OBJECT,
+                 G_TYPE_NONE,
+                 1,
+                 XFW_TYPE_SEAT);
+
     /**
      * XfwScreen::window-opened:
      * @screen: the object which received the signal.
@@ -360,6 +399,7 @@ static void
 xfw_screen_finalize(GObject *object) {
     XfwScreenPrivate *priv = XFW_SCREEN_GET_PRIVATE(object);
 
+    g_list_free_full(priv->seats, g_object_unref);
     g_list_free_full(priv->monitors, g_object_unref);
 
     g_object_unref(priv->workspace_manager);
@@ -367,6 +407,21 @@ xfw_screen_finalize(GObject *object) {
     G_OBJECT_CLASS(xfw_screen_parent_class)->finalize(object);
 }
 
+/**
+ * xfw_screen_get_seats:
+ * @screen: an #XfwScreen.
+ *
+ * Returns the list of all #XfwSeat instances available on @screen.
+ *
+ * Return value: (nullable) (element-type XfwSeat) (transfer none): a list of
+ * #XfwSeat.  The list and its contents are owned by @screen.
+ **/
+GList *
+xfw_screen_get_seats(XfwScreen *screen) {
+    g_return_val_if_fail(XFW_IS_SCREEN(screen), NULL);
+    return XFW_SCREEN_GET_PRIVATE(screen)->seats;
+}
+
 /**
  * xfw_screen_get_workspace_manager:
  * @screen: an #XfwScreen.
@@ -556,7 +611,7 @@ xfw_screen_get(GdkScreen *gdk_screen) {
 }
 
 /**
- * xfw_screen_get_default: (constructor)
+ * xfw_screen_get_default:
  *
  * Retrieves the #XfwScreen instance corresponding to the default #GdkScreen.
  *
@@ -574,6 +629,21 @@ _xfw_screen_get_gdk_screen(XfwScreen *screen) {
     return XFW_SCREEN_GET_PRIVATE(screen)->gdk_screen;
 }
 
+void
+_xfw_screen_seat_added(XfwScreen *screen, XfwSeat *seat) {
+    XfwScreenPrivate *priv = XFW_SCREEN_GET_PRIVATE(screen);
+    priv->seats = g_list_append(priv->seats, seat);
+    g_signal_emit_by_name(screen, "seat-added", seat);
+}
+
+void
+_xfw_screen_seat_removed(XfwScreen *screen, XfwSeat *seat) {
+    XfwScreenPrivate *priv = XFW_SCREEN_GET_PRIVATE(screen);
+    priv->seats = g_list_remove(priv->seats, seat);
+    g_signal_emit_by_name(screen, "seat-removed", seat);
+    g_object_unref(seat);
+}
+
 void
 _xfw_screen_set_workspace_manager(XfwScreen *screen, XfwWorkspaceManager *workspace_manager) {
     XFW_SCREEN_GET_PRIVATE(screen)->workspace_manager = workspace_manager;
@@ -641,3 +711,6 @@ _xfw_screen_set_show_desktop(XfwScreen *screen, gboolean show_desktop) {
         g_object_notify(G_OBJECT(screen), "show-desktop");
     }
 }
+
+#define __XFW_SCREEN_C__
+#include <libxfce4windowing-visibility.c>
