@@ -46,7 +46,7 @@ G_DEFINE_ENUM_TYPE(XfwDirection, xfw_direction,
                    G_DEFINE_ENUM_VALUE(XFW_DIRECTION_UP, "up"),
                    G_DEFINE_ENUM_VALUE(XFW_DIRECTION_DOWN, "down"),
                    G_DEFINE_ENUM_VALUE(XFW_DIRECTION_LEFT, "left"),
-                   G_DEFINE_ENUM_VALUE(XFW_DIRECTION_RIGHT, "right"));
+                   G_DEFINE_ENUM_VALUE(XFW_DIRECTION_RIGHT, "right"))
 
 /**
  * xfw_windowing_get:
