@@ -17,7 +17,9 @@
  * MA 02110-1301 USA
  */
 
+#ifdef HAVE_CONFIG_H
 #include "config.h"
+#endif
 
 #include <glib/gi18n-lib.h>
 
