@@ -24,7 +24,9 @@
 #error "Only libxfce4windowing.h can be included directly"
 #endif
 
+#include "xfw-monitor.h"
 #include "xfw-workspace-group.h"
+#include "xfw-workspace-manager.h"
 
 G_BEGIN_DECLS
 
@@ -41,9 +43,9 @@ struct _XfwWorkspaceGroupInterface {
     void (*active_workspace_changed)(XfwWorkspaceGroup *group,
                                      XfwWorkspace *previously_active_workspace);
     void (*monitor_added)(XfwWorkspaceGroup *group,
-                          GdkMonitor *monitor);
+                          XfwMonitor *monitor);
     void (*monitor_removed)(XfwWorkspaceGroup *group,
-                            GdkMonitor *monitor);
+                            XfwMonitor *monitor);
     void (*monitors_changed)(XfwWorkspaceGroup *group);
     void (*viewports_changed)(XfwWorkspaceGroup *group);
     void (*workspace_added)(XfwWorkspaceGroup *group,
@@ -66,4 +68,4 @@ struct _XfwWorkspaceGroupInterface {
 
 G_END_DECLS
 
-#endif  /* !__XFW_WORKSPACE_GROUP_PRIVATE_H__ */
+#endif /* !__XFW_WORKSPACE_GROUP_PRIVATE_H__ */
