@@ -73,7 +73,7 @@ static XfwWorkspaceGroup *xfw_workspace_wayland_get_workspace_group(XfwWorkspace
 static gint xfw_workspace_wayland_get_layout_row(XfwWorkspace *workspace);
 static gint xfw_workspace_wayland_get_layout_column(XfwWorkspace *workspace);
 static XfwWorkspace *xfw_workspace_wayland_get_neighbor(XfwWorkspace *workspace, XfwDirection direction);
-static GdkRectangle *xfw_workspace_x11_get_geometry(XfwWorkspace *workspace);
+static GdkRectangle *xfw_workspace_wayland_get_geometry(XfwWorkspace *workspace);
 static gboolean xfw_workspace_wayland_activate(XfwWorkspace *workspace, GError **error);
 static gboolean xfw_workspace_wayland_remove(XfwWorkspace *workspace, GError **error);
 static gboolean xfw_workspace_wayland_assign_to_workspace_group(XfwWorkspace *workspace, XfwWorkspaceGroup *group, GError **error);
@@ -148,7 +148,7 @@ xfw_workspace_wayland_workspace_init(XfwWorkspaceIface *iface) {
     iface->get_layout_row = xfw_workspace_wayland_get_layout_row;
     iface->get_layout_column = xfw_workspace_wayland_get_layout_column;
     iface->get_neighbor = xfw_workspace_wayland_get_neighbor;
-    iface->get_geometry = xfw_workspace_x11_get_geometry;
+    iface->get_geometry = xfw_workspace_wayland_get_geometry;
     iface->activate = xfw_workspace_wayland_activate;
     iface->remove = xfw_workspace_wayland_remove;
     iface->assign_to_workspace_group = xfw_workspace_wayland_assign_to_workspace_group;
@@ -162,15 +162,12 @@ xfw_workspace_wayland_set_property(GObject *obj, guint prop_id, const GValue *va
             workspace->priv->handle = g_value_get_pointer(value);
             break;
 
-        case WORKSPACE_PROP_GROUP:
-        case WORKSPACE_PROP_ID:
-            g_free(workspace->priv->id);
-            workspace->priv->id = g_value_dup_string(value);
-            break;
         case WORKSPACE_PROP_NAME:
             g_free(workspace->priv->name);
             workspace->priv->name = g_value_dup_string(value);
             break;
+        case WORKSPACE_PROP_GROUP:
+        case WORKSPACE_PROP_ID:
         case WORKSPACE_PROP_CAPABILITIES:
         case WORKSPACE_PROP_STATE:
         case WORKSPACE_PROP_NUMBER:
@@ -312,7 +309,7 @@ xfw_workspace_wayland_get_neighbor(XfwWorkspace *workspace, XfwDirection directi
 }
 
 static GdkRectangle *
-xfw_workspace_x11_get_geometry(XfwWorkspace *workspace) {
+xfw_workspace_wayland_get_geometry(XfwWorkspace *workspace) {
     // probably something to do with coordinates and outputs if needed
     return &XFW_WORKSPACE_WAYLAND(workspace)->priv->geometry;
 }
@@ -369,7 +366,9 @@ xfw_workspace_wayland_assign_to_workspace_group(XfwWorkspace *workspace, XfwWork
 static void
 workspace_id(void *data, struct ext_workspace_handle_v1 *wl_workspace, const char *id) {
     XfwWorkspaceWayland *workspace = XFW_WORKSPACE_WAYLAND(data);
-    g_object_set(workspace, "id", id, NULL);
+    g_free(workspace->priv->id);
+    workspace->priv->id = g_strdup(id);
+    g_object_notify(G_OBJECT(workspace), "id");
 }
 
 static void
